from django_inlines.inlines import TemplateInline, InlineUnrenderableError
from esv import esv, PassageNotFoundError, EsvQuotaExceededError


class InlineQuotaExceededError(InlineUnrenderableError):
    pass

class InlinePassageNotFoundError(InlineUnrenderableError):
    pass


class PassageInline(TemplateInline):
    """
    An inline for retrieving passages from the ESV REST webservice.
    
    Takes any passage or passages and returns the ESV (x)html output for that 
    passage. ::
    
        {{ passage reference [arguments] }}
    
    Takes these arguments::
    
        headings=on/off default: off
        audio=on/off default: on
        footnotes=on/off default: off
    
    examples::
    
        {{ passage John 1 }}
        {{ passage John 2:1-3:18 footnotes=on }}
        {{ passage jhn 2 matt 3 }}
    
    """
    def __init__(self, value, footnotes='off', audio='on', headings='off', **kwargs):
        super(PassageInline, self).__init__(value, **kwargs)
        on_off_map = { 'on': True, 'off': False }
        self.footnotes = on_off_map[footnotes]
        self.audio = on_off_map[audio]
        self.headings = on_off_map[headings]
    
    def get_context(self):
        try:
            passage = esv.get_passage(self.value, footnotes=self.footnotes, headings=self.headings, audio=self.audio)
            return {'passage': passage }
        except PassageNotFoundError:
            raise InlinePassageNotFoundError
        except EsvQuotaExceededError:
            raise InlineQuotaExceededError



import urllib
import httplib2
from django.conf import settings

class EsvClientError(Exception):
    pass

class PassageNotFoundError(EsvClientError):
    pass

class EsvQuotaExceededError(EsvClientError):
    pass


class EsvClient(object):
    def __init__(self, key='IP'):
        http_cache = getattr(settings, 'ESV_HTTP_CACHE', '/tmp/esv_http_cache')
        self.http = httplib2.Http(http_cache)
        self.key = key
        self._cache = {}
    
    def get_passage(self, passage, headings=False, audio=True, footnotes=False, audio_format="flash"):
        params_dict = {
            'passage': passage,
            'include-headings': headings,
            'include_footnotes': footnotes,
            'include-word-ids': False,
            'include-first-verse-numbers': False,
            'include-audio-link': audio,
            'audio-format': audio_format,
        }
        params = urllib.urlencode(params_dict).lower()
        # TODO: Check cache here
        resp, content = self.http.request("http://www.esvapi.org/v2/rest/passageQuery?key=%s&%s" % (self.key, params), "GET")
        if content.startswith("ERROR"):
            if content.lower().find('no results found') > 0:
                raise PassageNotFoundError
            if content.lower().find('you have exceeded your quota') > 0:
                raise EsvQuotaExceededError
            raise EsvClientError
        # TODO: Set cache here
        return content


# main instance of the esv client
esv = EsvClient()


from esv import esv, PassageNotFoundError, EsvQuotaExceededError
from django import template
from django.conf import settings


register = template.Library()


class PassageNode(template.Node):
    
    def __init__(self, reference, footnotes='off', audio='on', headings='off', **kwargs):
        on_off_map = { 'on': True, 'off': False }
        self.reference = template.Variable(reference)
        self.footnotes = on_off_map[footnotes]
        self.audio = on_off_map[audio]
        self.headings = on_off_map[headings]
        
    def render(self, context):
        passage = ""
        try:
            passage = esv.get_passage(self.reference.resolve(context), footnotes=self.footnotes, headings=self.headings, audio=self.audio)
        except (PassageNotFoundError, EsvQuotaExceededError):
            if settings.TEMPLATE_DEBUG:
                raise
        return passage


@register.tag
def passage(parser, token):
    """
    Looks up the specified reference and returns passage from ESV's bible API
    
    Syntax::
    
      {% passage reference [headings on] [audio off] [footnotes on] %}

    Where reference is a string that the ESV can use as a query, or an context 
    var that resolves to such a string.

    Examples::

      {% passage "Genesis 1:1" %}
      {% passage "rom 3" %}
      {% passage "1 tim 3-4" footnotes on %}
    
    """
    ALLOWED_ARGS = ['headings', 'footnotes', 'audio']
    ALLOWED_VALUES = ['on', 'off']
    
    args = token.split_contents()
    
    if not len(args) in (2, 4, 6, 8):
        raise template.TemplateSyntaxError("%r tag requires either 1, 3, 5 or 7 arguments." % args[0])
    
    reference = args[1]
    kwargs = {}
    if len(args) > 2:
        tuples = zip(*[args[2:][i::2] for i in range(2)])
        for k,v in tuples:
            if not k in ALLOWED_ARGS:
                raise template.TemplateSyntaxError("%r tag options arguments must be one of %s." % (args[0], ', '.join(ALLOWED_ARGS)))
            if not v in ALLOWED_VALUES:
                raise template.TemplateSyntaxError("the values for %r's arguments must be one of %s." % (args[0], ', '.join(ALLOWED_VALUES)))
            kwargs[str(k)] = str(v)

    return PassageNode(reference, **kwargs)



import os
from distutils.core import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

README = read('README.rst')

setup(
    name = "django_esv",
    version = "0.8",
    url = 'http://github.com/mintchaos/django_es',
    license = 'BSD',
    description = "A django_inlines style inline and a template tag to work with the ESV Bible API.",
    long_description=README,
    
    author = 'Christian Metts',
    author_email = 'xian@mintchaos.com',
    packages = [
        'esv',
        'esv.templatetags',
    ],
    install_requires = ['httplib2'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)
